@@ -12,15 +12,17 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "dialogs/dialogs_key.h"
 #include "dialogs/dialogs_entry.h"
 #include "history/history.h"
-//#include "history/feed/history_feed_section.h" // #feed
 #include "history/view/history_view_top_bar_widget.h"
 #include "ui/widgets/buttons.h"
 #include "ui/widgets/input_fields.h"
 #include "ui/wrap/fade_wrap.h"
 #include "ui/effects/radial_animation.h"
+#include "ui/controls/download_bar.h"
+#include "ui/ui_utility.h"
 #include "lang/lang_keys.h"
 #include "mainwindow.h"
 #include "mainwidget.h"
+#include "main/main_domain.h"
 #include "main/main_session.h"
 #include "main/main_session_settings.h"
 #include "apiwrap.h"
@@ -29,12 +31,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "core/update_checker.h"
 #include "boxes/peer_list_box.h"
 #include "boxes/peers/edit_participants_box.h"
+#include "window/window_adaptive.h"
+#include "window/window_controller.h"
 #include "window/window_session_controller.h"
 #include "window/window_slide_animation.h"
 #include "window/window_connecting_widget.h"
 #include "window/window_main_menu.h"
 #include "storage/storage_media_prepare.h"
 #include "storage/storage_account.h"
+#include "storage/storage_domain.h"
 #include "data/data_session.h"
 #include "data/data_channel.h"
 #include "data/data_chat.h"
@@ -42,12 +47,15 @@ https://github.com/telegramdesktop/tdesktop/blob/master/LEGAL
 #include "data/data_folder.h"
 #include "data/data_histories.h"
 #include "data/data_changes.h"
+#include "data/data_download_manager.h"
+#include "info/downloads/info_downloads_widget.h"
+#include "info/info_memento.h"
 #include "facades.h"
-#include "app.h"
 #include "styles/style_dialogs.h"
 #include "styles/style_chat.h"
 #include "styles/style_info.h"
 #include "styles/style_window.h"
+#include "base/qt/qt_common_adapters.h"
 
 #include <QtCore/QMimeData>
 
@@ -161,8 +169,9 @@ void Widget::BottomButton::paintEvent(QPaintEvent *e) {
 Widget::Widget(
 	QWidget *parent,
 	not_null<Window::SessionController*> controller)
-	: Window::AbstractSectionWidget(parent, controller)
+: Window::AbstractSectionWidget(parent, controller, nullptr)
 , _api(&controller->session().mtp())
+, _chooseByDragTimer([=] { _inner->chooseRow(); })
 , _searchControls(this)
 , _mainMenuToggle(_searchControls, st::dialogsMenuToggle)
 , _searchForNarrowFilters(_searchControls, st::dialogsSearchForNarrowFilters)
@@ -175,14 +184,15 @@ Widget::Widget(
 	object_ptr<Ui::IconButton>(this, st::dialogsCalendar))
 , _cancelSearch(_searchControls, st::dialogsCancelSearch)
 , _lockUnlock(_searchControls, st::dialogsLock)
-, _scroll(this, st::dialogsScroll)
+, _scroll(this)
 , _scrollToTop(_scroll, st::dialogsToUp)
+, _searchTimer([=] { searchMessages(); })
 , _singleMessageSearch(&controller->session()) {
 	_inner = _scroll->setOwnedWidget(object_ptr<InnerWidget>(this, controller));
 
 	_inner->updated(
 	) | rpl::start_with_next([=] {
-		onListScroll();
+		listScrollUpdated();
 	}, lifetime());
 
 	rpl::combine(
@@ -201,27 +211,72 @@ Widget::Widget(
 	fullSearchRefreshOn(session().settings().skipArchiveInSearchChanges(
 	) | rpl::to_empty);
 
-	connect(_inner, SIGNAL(draggingScrollDelta(int)), this, SLOT(onDraggingScrollDelta(int)));
-	connect(_inner, SIGNAL(mustScrollTo(int,int)), _scroll, SLOT(scrollToY(int,int)));
-	connect(_inner, SIGNAL(dialogMoved(int,int)), this, SLOT(onDialogMoved(int,int)));
-	connect(_inner, SIGNAL(searchMessages()), this, SLOT(onNeedSearchMessages()));
-	connect(_inner, SIGNAL(completeHashtag(QString)), this, SLOT(onCompleteHashtag(QString)));
-	connect(_inner, SIGNAL(refreshHashtags()), this, SLOT(onFilterCursorMoved()));
-	connect(_inner, SIGNAL(cancelSearchInChat()), this, SLOT(onCancelSearchInChat()));
-	subscribe(_inner->searchFromUserChanged, [this](PeerData *from) {
-		setSearchInChat(_searchInChat, from);
+	_inner->scrollByDeltaRequests(
+	) | rpl::start_with_next([=](int delta) {
+		if (_scroll) {
+			_scroll->scrollToY(_scroll->scrollTop() + delta);
+		}
+	}, lifetime());
+
+	_inner->mustScrollTo(
+	) | rpl::start_with_next([=](const Ui::ScrollToRequest &data) {
+		if (_scroll) {
+			_scroll->scrollToY(data.ymin, data.ymax);
+		}
+	}, lifetime());
+	_inner->dialogMoved(
+	) | rpl::start_with_next([=](const Ui::ScrollToRequest &data) {
+		const auto movedFrom = data.ymin;
+		const auto movedTo = data.ymax;
+		const auto st = int32(_scroll->scrollTop());
+		if (st > movedTo && st < movedFrom) {
+			_scroll->scrollToY(st + st::dialogsRowHeight);
+		}
+	}, lifetime());
+	_inner->searchMessages(
+	) | rpl::start_with_next([=] {
+		needSearchMessages();
+	}, lifetime());
+	_inner->cancelSearchInChatRequests(
+	) | rpl::start_with_next([=] {
+		cancelSearchInChat();
+	}, lifetime());
+	_inner->completeHashtagRequests(
+	) | rpl::start_with_next([=](const QString &tag) {
+		completeHashtag(tag);
+	}, lifetime());
+	_inner->refreshHashtagsRequests(
+	) | rpl::start_with_next([=] {
+		filterCursorMoved();
+	}, lifetime());
+	_inner->cancelSearchFromUserRequests(
+	) | rpl::start_with_next([=] {
+		setSearchInChat(_searchInChat, nullptr);
 		applyFilterUpdate(true);
-	});
+	}, lifetime());
 	_inner->chosenRow(
 	) | rpl::start_with_next([=](const ChosenRow &row) {
 		const auto openSearchResult = !controller->selectingPeer()
 			&& row.filteredRow;
 		if (const auto history = row.key.history()) {
-			controller->content()->choosePeer(
-				history->peer->id,
-				(controller->uniqueChatsInSearchResults()
-					? ShowAtUnreadMsgId
-					: row.message.fullId.msg));
+			const auto peer = history->peer;
+			const auto showAtMsgId = controller->uniqueChatsInSearchResults()
+				? ShowAtUnreadMsgId
+				: row.message.fullId.msg;
+			if (row.newWindow) {
+				const auto active = controller->activeChatCurrent();
+				if (const auto history = active.history()) {
+					if (history->peer == peer) {
+						controller->content()->ui_showPeerHistory(
+							0,
+							Window::SectionShow::Way::ClearStack,
+							0);
+					}
+				}
+				Core::App().ensureSeparateWindowForPeer(peer, showAtMsgId);
+			} else {
+				controller->content()->choosePeer(peer->id, showAtMsgId);
+			}
 		} else if (const auto folder = row.key.folder()) {
 			controller->openFolder(folder);
 		}
@@ -230,31 +285,37 @@ Widget::Widget(
 		}
 	}, lifetime());
 
-	connect(_scroll, SIGNAL(geometryChanged()), _inner, SLOT(onParentGeometryChanged()));
-	connect(_scroll, SIGNAL(scrolled()), this, SLOT(onListScroll()));
+	_scroll->geometryChanged(
+	) | rpl::start_with_next(crl::guard(_inner, [=] {
+		_inner->parentGeometryChanged();
+	}), lifetime());
+	_scroll->scrolls(
+	) | rpl::start_with_next([=] {
+		listScrollUpdated();
+	}, lifetime());
 
 	session().data().chatsListChanges(
 	) | rpl::filter([=](Data::Folder *folder) {
 		return (folder == _inner->shownFolder());
 	}) | rpl::start_with_next([=] {
-		Ui::PostponeCall(this, [=] { onListScroll(); });
+		Ui::PostponeCall(this, [=] { listScrollUpdated(); });
 	}, lifetime());
 
-	connect(_filter, &Ui::FlatInput::cancelled, [=] {
+	QObject::connect(_filter, &Ui::FlatInput::cancelled, [=] {
 		escape();
 	});
-	connect(_filter, &Ui::FlatInput::changed, [=] {
+	QObject::connect(_filter, &Ui::FlatInput::changed, [=] {
 		applyFilterUpdate();
 	});
-	connect(
+	QObject::connect(
 		_filter,
 		&Ui::FlatInput::cursorPositionChanged,
-		[=](int from, int to) { onFilterCursorMoved(from, to); });
+		[=](int from, int to) { filterCursorMoved(from, to); });
 
 	if (!Core::UpdaterDisabled()) {
 		Core::UpdateChecker checker;
 		rpl::merge(
-			rpl::single(rpl::empty_value()),
+			rpl::single(rpl::empty),
 			checker.isLatest(),
 			checker.failed(),
 			checker.ready()
@@ -263,13 +324,19 @@ Widget::Widget(
 		}, lifetime());
 	}
 
-	subscribe(Adaptive::Changed(), [this] { updateForwardBar(); });
+	controller->adaptive().changes(
+	) | rpl::start_with_next([=] {
+		updateForwardBar();
+	}, lifetime());
 
-	_cancelSearch->setClickedCallback([this] { onCancelSearch(); });
-	_jumpToDate->entity()->setClickedCallback([this] { showJumpToDate(); });
+	_cancelSearch->setClickedCallback([this] { cancelSearch(); });
+	_jumpToDate->entity()->setClickedCallback([this] { showCalendar(); });
 	_chooseFromUser->entity()->setClickedCallback([this] { showSearchFrom(); });
-	_lockUnlock->setVisible(Global::LocalPasscode());
-	subscribe(Global::RefLocalPasscodeChanged(), [this] { updateLockUnlockVisibility(); });
+	rpl::single(rpl::empty) | rpl::then(
+		session().domain().local().localPasscodeChanged()
+	) | rpl::start_with_next([=] {
+		updateLockUnlockVisibility();
+	}, lifetime());
 	_lockUnlock->setClickedCallback([this] {
 		_lockUnlock->setIconOverride(&st::dialogsUnlockIcon, &st::dialogsUnlockIconOver);
 		Core::App().lockByPasscode();
@@ -280,14 +347,8 @@ Widget::Widget(
 
 	_searchForNarrowFilters->setClickedCallback([=] { Ui::showChatsList(&session()); });
 
-	_chooseByDragTimer.setSingleShot(true);
-	connect(&_chooseByDragTimer, SIGNAL(timeout()), this, SLOT(onChooseByDrag()));
-
 	setAcceptDrops(true);
 
-	_searchTimer.setSingleShot(true);
-	connect(&_searchTimer, SIGNAL(timeout()), this, SLOT(onSearchMessages()));
-
 	_inner->setLoadMoreCallback([=] {
 		const auto state = _inner->state();
 		if (state == WidgetState::Filtered
@@ -295,7 +356,7 @@ Widget::Widget(
 				|| (_searchInMigrated
 					&& _searchFull
 					&& !_searchFullMigrated))) {
-			onSearchMore();
+			searchMore();
 		} else {
 			const auto folder = _inner->shownFolder();
 			if (!folder || !folder->chatsList()->loaded()) {
@@ -325,6 +386,8 @@ Widget::Widget(
 	) | rpl::start_with_next([=](Data::Folder *folder) {
 		changeOpenedFolder(folder, anim::type::normal);
 	}, lifetime());
+
+	setupDownloadBar();
 }
 
 void Widget::setGeometryWithTopMoved(
@@ -363,6 +426,61 @@ void Widget::setupScrollUpButton() {
 	updateScrollUpVisibility();
 }
 
+void Widget::setupDownloadBar() {
+	Data::MakeDownloadBarContent(
+	) | rpl::start_with_next([=](Ui::DownloadBarContent &&content) {
+		const auto create = (content.count && !_downloadBar);
+		if (create) {
+			_downloadBar = std::make_unique<Ui::DownloadBar>(
+				this,
+				Data::MakeDownloadBarProgress());
+		}
+		if (_downloadBar) {
+			_downloadBar->show(std::move(content));
+		}
+		if (create) {
+			_downloadBar->heightValue(
+			) | rpl::start_with_next([=] {
+				updateControlsGeometry();
+			}, _downloadBar->lifetime());
+
+			_downloadBar->shownValue(
+			) | rpl::filter(
+				!rpl::mappers::_1
+			) | rpl::start_with_next([=] {
+				_downloadBar = nullptr;
+				updateControlsGeometry();
+			}, _downloadBar->lifetime());
+
+			_downloadBar->clicks(
+			) | rpl::start_with_next([=] {
+				auto &&list = Core::App().downloadManager().loadingList();
+				const auto guard = gsl::finally([] {
+					Core::App().downloadManager().clearIfFinished();
+				});
+				auto first = (HistoryItem*)nullptr;
+				for (const auto id : list) {
+					if (!first) {
+						first = id->object.item;
+					} else {
+						controller()->showSection(
+							Info::Downloads::Make(
+								controller()->session().user()));
+						return;
+					}
+				}
+				if (first) {
+					controller()->showPeerHistoryAtItem(first);
+				}
+			}, _downloadBar->lifetime());
+
+			if (_connecting) {
+				_connecting->raise();
+			}
+		}
+	}, lifetime());
+}
+
 void Widget::updateScrollUpVisibility() {
 	if (_scrollToAnimation.animating()) {
 		return;
@@ -407,7 +525,7 @@ void Widget::setupConnectingWidget() {
 	_connecting = std::make_unique<Window::ConnectionState>(
 		this,
 		&session().account(),
-		Window::AdaptiveIsOneColumn());
+		controller()->adaptive().oneColumnValue());
 }
 
 void Widget::setupSupportMode() {
@@ -422,9 +540,7 @@ void Widget::setupSupportMode() {
 void Widget::setupMainMenuToggle() {
 	_mainMenuToggle->setClickedCallback([=] { showMainMenu(); });
 
-	rpl::single(
-		rpl::empty_value()
-	) | rpl::then(
+	rpl::single(rpl::empty) | rpl::then(
 		controller()->filtersMenuChanged()
 	) | rpl::start_with_next([=] {
 		const auto filtersHidden = !controller()->filtersWidth();
@@ -450,7 +566,7 @@ void Widget::fullSearchRefreshOn(rpl::producer<> events) {
 	) | rpl::filter([=] {
 		return !_searchQuery.isEmpty();
 	}) | rpl::start_with_next([=] {
-		_searchTimer.stop();
+		_searchTimer.cancel();
 		_searchCache.clear();
 		_singleMessageSearch.clear();
 		for (const auto &[requestId, query] : base::take(_searchQueries)) {
@@ -459,7 +575,7 @@ void Widget::fullSearchRefreshOn(rpl::producer<> events) {
 		_searchQuery = QString();
 		_scroll->scrollToY(0);
 		cancelSearchRequest();
-		onSearchMessages();
+		searchMessages();
 	}, lifetime());
 }
 
@@ -558,7 +674,9 @@ void Widget::checkUpdateStatus() {
 
 	using Checker = Core::UpdateChecker;
 	if (Checker().state() == Checker::State::Ready) {
-		if (_updateTelegram) return;
+		if (_updateTelegram) {
+			return;
+		}
 		_updateTelegram.create(
 			this,
 			tr::lng_update_telegram(tr::now),
@@ -568,10 +686,15 @@ void Widget::checkUpdateStatus() {
 		_updateTelegram->show();
 		_updateTelegram->setClickedCallback([] {
 			Core::checkReadyUpdate();
-			App::restart();
+			Core::Restart();
 		});
+		if (_connecting) {
+			_connecting->raise();
+		}
 	} else {
-		if (!_updateTelegram) return;
+		if (!_updateTelegram) {
+			return;
+		}
 		_updateTelegram.destroy();
 	}
 	updateControlsGeometry();
@@ -640,7 +763,7 @@ void Widget::startWidthAnimation() {
 		QPainter p(&image);
 		Ui::RenderWidget(p, _scroll);
 	}
-	_widthAnimationCache = App::pixmapFromImageInPlace(std::move(image));
+	_widthAnimationCache = Ui::PixmapFromImage(std::move(image));
 	_scroll->setGeometry(scrollGeometry);
 	_scroll->hide();
 }
@@ -688,7 +811,6 @@ void Widget::startSlideAnimation() {
 		_folderTopBar->hide();
 	}
 
-	int delta = st::slideShift;
 	if (_showDirection == Window::SlideDirection::FromLeft) {
 		std::swap(_cacheUnder, _cacheOver);
 	}
@@ -719,15 +841,15 @@ void Widget::animationCallback() {
 void Widget::escape() {
 	if (controller()->openedFolder().current()) {
 		controller()->closeFolder();
-	} else if (!onCancelSearch()) {
+	} else if (!cancelSearch()) {
 		if (controller()->activeChatEntryCurrent().key) {
-			emit cancelled();
+			controller()->content()->dialogsCancelled();
 		} else if (controller()->activeChatsFilterCurrent()) {
 			controller()->setActiveChatsFilter(FilterId(0));
 		}
 	} else if (!_searchInChat && !controller()->selectingPeer()) {
 		if (controller()->activeChatEntryCurrent().key) {
-			emit cancelled();
+			controller()->content()->dialogsCancelled();
 		}
 	}
 }
@@ -767,28 +889,7 @@ void Widget::loadMoreBlockedByDate() {
 	session().api().requestMoreBlockedByDateDialogs();
 }
 
-void Widget::onDraggingScrollDelta(int delta) {
-	_draggingScrollDelta = _scroll ? delta : 0;
-	if (_draggingScrollDelta) {
-		if (!_draggingScrollTimer) {
-			_draggingScrollTimer.create(this);
-			_draggingScrollTimer->setSingleShot(false);
-			connect(_draggingScrollTimer, SIGNAL(timeout()), this, SLOT(onDraggingScrollTimer()));
-		}
-		_draggingScrollTimer->start(15);
-	} else {
-		_draggingScrollTimer.destroy();
-	}
-}
-
-void Widget::onDraggingScrollTimer() {
-	const auto delta = (_draggingScrollDelta > 0)
-		? qMin(_draggingScrollDelta * 3 / 20 + 1, Ui::kMaxScrollSpeed)
-		: qMax(_draggingScrollDelta * 3 / 20 - 1, -Ui::kMaxScrollSpeed);
-	_scroll->scrollToY(_scroll->scrollTop() + delta);
-}
-
-bool Widget::onSearchMessages(bool searchCache) {
+bool Widget::searchMessages(bool searchCache) {
 	auto result = false;
 	auto q = _filter->getLastText().trimmed();
 	if (q.isEmpty() && !_searchFromAuthor) {
@@ -798,7 +899,7 @@ bool Widget::onSearchMessages(bool searchCache) {
 	}
 	if (searchCache) {
 		const auto success = _singleMessageSearch.lookup(q, [=] {
-			onNeedSearchMessages();
+			needSearchMessages();
 		});
 		if (!success) {
 			return false;
@@ -842,19 +943,19 @@ bool Widget::onSearchMessages(bool searchCache) {
 						: MTP_inputPeerEmpty()),
 					MTPint(), // top_msg_id
 					MTP_inputMessagesFilterEmpty(),
-					MTP_int(0),
-					MTP_int(0),
-					MTP_int(0),
-					MTP_int(0),
+					MTP_int(0), // min_date
+					MTP_int(0), // max_date
+					MTP_int(0), // offset_id
+					MTP_int(0), // add_offset
 					MTP_int(SearchPerPage),
-					MTP_int(0),
-					MTP_int(0),
-					MTP_int(0)
+					MTP_int(0), // max_id
+					MTP_int(0), // min_id
+					MTP_long(0) // hash
 				)).done([=](const MTPmessages_Messages &result) {
 					_searchInHistoryRequest = 0;
 					searchReceived(type, result, _searchRequest);
 					finish();
-				}).fail([=](const RPCError &error) {
+				}).fail([=](const MTP::Error &error) {
 					_searchInHistoryRequest = 0;
 					searchFailed(type, error, _searchRequest);
 					finish();
@@ -862,21 +963,6 @@ bool Widget::onSearchMessages(bool searchCache) {
 				_searchQueries.emplace(_searchRequest, _searchQuery);
 				return _searchRequest;
 			});
-		//} else if (const auto feed = _searchInChat.feed()) { // #feed
-		//	const auto type = SearchRequestType::FromStart;
-		//	_searchRequest = session().api().request(MTPchannels_SearchFeed(
-		//		MTP_int(feed->id()),
-		//		MTP_string(_searchQuery),
-		//		MTP_int(0),
-		//		MTP_inputPeerEmpty(),
-		//		MTP_int(0),
-		//		MTP_int(SearchPerPage)
-		//	)).done([=](const MTPmessages_Messages &result) {
-		//		searchReceived(type, result, _searchRequest);
-		//	}).fail([=](const RPCError &error) {
-		//		searchFailed(type, error, _searchRequest);
-		//	}).send();
-		//	_searchQueries.emplace(_searchRequest, _searchQuery);
 		} else {
 			const auto type = SearchRequestType::FromStart;
 			const auto flags = session().settings().skipArchiveInSearch()
@@ -896,7 +982,7 @@ bool Widget::onSearchMessages(bool searchCache) {
 				MTP_int(SearchPerPage)
 			)).done([=](const MTPmessages_Messages &result) {
 				searchReceived(type, result, _searchRequest);
-			}).fail([=](const RPCError &error) {
+			}).fail([=](const MTP::Error &error) {
 				searchFailed(type, error, _searchRequest);
 			}).send();
 			_searchQueries.emplace(_searchRequest, _searchQuery);
@@ -920,7 +1006,7 @@ bool Widget::onSearchMessages(bool searchCache) {
 				MTP_int(SearchPeopleLimit)
 			)).done([=](const MTPcontacts_Found &result, mtpRequestId requestId) {
 				peerSearchReceived(result, requestId);
-			}).fail([=](const RPCError &error, mtpRequestId requestId) {
+			}).fail([=](const MTP::Error &error, mtpRequestId requestId) {
 				peopleFailed(error, requestId);
 			}).send();
 			_peerSearchQueries.emplace(_peerSearchRequest, _peerSearchQuery);
@@ -946,16 +1032,12 @@ bool Widget::searchForPeersRequired(const QString &query) const {
 	return (query[0] != '#');
 }
 
-void Widget::onNeedSearchMessages() {
-	if (!onSearchMessages(true)) {
-		_searchTimer.start(AutoSearchTimeout);
+void Widget::needSearchMessages() {
+	if (!searchMessages(true)) {
+		_searchTimer.callOnce(AutoSearchTimeout);
 	}
 }
 
-void Widget::onChooseByDrag() {
-	_inner->chooseRow();
-}
-
 void Widget::showMainMenu() {
 	controller()->widget()->showMainMenu();
 }
@@ -975,20 +1057,20 @@ void Widget::searchMessages(
 	}();
 	if ((_filter->getLastText() != query) || inChatChanged) {
 		if (inChat) {
-			onCancelSearch();
+			cancelSearch();
 			setSearchInChat(inChat);
 		}
 		_filter->setText(query);
 		_filter->updatePlaceholder();
 		applyFilterUpdate(true);
-		_searchTimer.stop();
-		onSearchMessages();
+		_searchTimer.cancel();
+		searchMessages();
 
 		session().local().saveRecentSearchHashtags(query);
 	}
 }
 
-void Widget::onSearchMore() {
+void Widget::searchMore() {
 	if (_searchRequest || _searchInHistoryRequest) {
 		return;
 	}
@@ -1015,19 +1097,19 @@ void Widget::onSearchMore() {
 						: MTP_inputPeerEmpty()),
 					MTPint(), // top_msg_id
 					MTP_inputMessagesFilterEmpty(),
-					MTP_int(0),
-					MTP_int(0),
+					MTP_int(0), // min_date
+					MTP_int(0), // max_date
 					MTP_int(offsetId),
-					MTP_int(0),
+					MTP_int(0), // add_offset
 					MTP_int(SearchPerPage),
-					MTP_int(0),
-					MTP_int(0),
-					MTP_int(0)
+					MTP_int(0), // max_id
+					MTP_int(0), // min_id
+					MTP_long(0) // hash
 				)).done([=](const MTPmessages_Messages &result) {
 					searchReceived(type, result, _searchRequest);
 					_searchInHistoryRequest = 0;
 					finish();
-				}).fail([=](const RPCError &error) {
+				}).fail([=](const MTP::Error &error) {
 					searchFailed(type, error, _searchRequest);
 					_searchInHistoryRequest = 0;
 					finish();
@@ -1037,27 +1119,6 @@ void Widget::onSearchMore() {
 				}
 				return _searchRequest;
 			});
-		//} else if (const auto feed = _searchInChat.feed()) { // #feed
-		//	const auto type = offsetId
-		//		? SearchRequestType::FromOffset
-		//		: SearchRequestType::FromStart;
-		//	_searchRequest = session().api().request(MTPchannels_SearchFeed(
-		//		MTP_int(feed->id()),
-		//		MTP_string(_searchQuery),
-		//		MTP_int(offsetDate),
-		//		offsetPeer
-		//			? offsetPeer->input
-		//			: MTP_inputPeerEmpty(),
-		//		MTP_int(offsetId),
-		//		MTP_int(SearchPerPage)),
-		//	)).done([=](const MTPmessages_Messages &result) {
-		//		searchReceived(type, result, _searchRequest);
-		//	}).fail([=](const RPCError &error) {
-		//		searchFailed(type, error, _searchRequest);
-		//	}).send();
-		//	if (!offsetId) {
-		//		_searchQueries.emplace(_searchRequest, _searchQuery);
-		//	}
 		} else {
 			const auto type = offsetId
 				? SearchRequestType::FromOffset
@@ -1081,7 +1142,7 @@ void Widget::onSearchMore() {
 				MTP_int(SearchPerPage)
 			)).done([=](const MTPmessages_Messages &result) {
 				searchReceived(type, result, _searchRequest);
-			}).fail([=](const RPCError &error) {
+			}).fail([=](const MTP::Error &error) {
 				searchFailed(type, error, _searchRequest);
 			}).send();
 			if (!offsetId) {
@@ -1109,19 +1170,19 @@ void Widget::onSearchMore() {
 					: MTP_inputPeerEmpty()),
 				MTPint(), // top_msg_id
 				MTP_inputMessagesFilterEmpty(),
-				MTP_int(0),
-				MTP_int(0),
+				MTP_int(0), // min_date
+				MTP_int(0), // max_date
 				MTP_int(offsetMigratedId),
-				MTP_int(0),
+				MTP_int(0), // add_offset
 				MTP_int(SearchPerPage),
-				MTP_int(0),
-				MTP_int(0),
-				MTP_int(0)
+				MTP_int(0), // max_id
+				MTP_int(0), // min_id
+				MTP_long(0) // hash
 			)).done([=](const MTPmessages_Messages &result) {
 				searchReceived(type, result, _searchRequest);
 				_searchInHistoryRequest = 0;
 				finish();
-			}).fail([=](const RPCError &error) {
+			}).fail([=](const MTP::Error &error) {
 				searchFailed(type, error, _searchRequest);
 				_searchInHistoryRequest = 0;
 				finish();
@@ -1237,7 +1298,7 @@ void Widget::searchReceived(
 	}
 
 	_searchRequest = 0;
-	onListScroll();
+	listScrollUpdated();
 	update();
 }
 
@@ -1264,13 +1325,13 @@ void Widget::peerSearchReceived(
 		}
 
 		_peerSearchRequest = 0;
-		onListScroll();
+		listScrollUpdated();
 	}
 }
 
 void Widget::searchFailed(
 		SearchRequestType type,
-		const RPCError &error,
+		const MTP::Error &error,
 		mtpRequestId requestId) {
 	if (error.type() == qstr("SEARCH_QUERY_EMPTY")) {
 		searchReceived(
@@ -1290,7 +1351,7 @@ void Widget::searchFailed(
 	}
 }
 
-void Widget::peopleFailed(const RPCError &error, mtpRequestId requestId) {
+void Widget::peopleFailed(const MTP::Error &error, mtpRequestId requestId) {
 	if (_peerSearchRequest == requestId) {
 		_peerSearchRequest = 0;
 		_peerSearchFull = true;
@@ -1306,7 +1367,7 @@ void Widget::dragEnterEvent(QDragEnterEvent *e) {
 
 	const auto data = e->mimeData();
 	_dragInScroll = false;
-	_dragForward = Adaptive::OneColumn()
+	_dragForward = controller()->adaptive().isOneColumn()
 		? false
 		: data->hasFormat(qsl("application/x-td-forward"));
 	if (_dragForward) {
@@ -1317,7 +1378,7 @@ void Widget::dragEnterEvent(QDragEnterEvent *e) {
 		e->setDropAction(Qt::CopyAction);
 		e->accept();
 	}
-	_chooseByDragTimer.stop();
+	_chooseByDragTimer.cancel();
 }
 
 void Widget::dragMoveEvent(QDragMoveEvent *e) {
@@ -1325,7 +1386,7 @@ void Widget::dragMoveEvent(QDragMoveEvent *e) {
 		if (_dragForward) {
 			updateDragInScroll(true);
 		} else {
-			_chooseByDragTimer.start(ChoosePeerByDragTimeout);
+			_chooseByDragTimer.callOnce(ChoosePeerByDragTimeout);
 		}
 		if (_inner->updateFromParentDrag(mapToGlobal(e->pos()))) {
 			e->setDropAction(Qt::CopyAction);
@@ -1344,7 +1405,7 @@ void Widget::dragLeaveEvent(QDragLeaveEvent *e) {
 	if (_dragForward) {
 		updateDragInScroll(false);
 	} else {
-		_chooseByDragTimer.stop();
+		_chooseByDragTimer.cancel();
 	}
 	_inner->dragLeft();
 	e->accept();
@@ -1362,19 +1423,20 @@ void Widget::updateDragInScroll(bool inScroll) {
 }
 
 void Widget::dropEvent(QDropEvent *e) {
-	_chooseByDragTimer.stop();
+	_chooseByDragTimer.cancel();
 	if (_scroll->geometry().contains(e->pos())) {
 		if (auto peer = _inner->updateFromParentDrag(mapToGlobal(e->pos()))) {
 			e->acceptProposedAction();
 			controller()->content()->onFilesOrForwardDrop(
 				peer->id,
 				e->mimeData());
+			controller()->widget()->raise();
 			controller()->widget()->activateWindow();
 		}
 	}
 }
 
-void Widget::onListScroll() {
+void Widget::listScrollUpdated() {
 	const auto scrollTop = _scroll->scrollTop();
 	_inner->setVisibleTopBottom(scrollTop, scrollTop + _scroll->height());
 	updateScrollUpVisibility();
@@ -1417,7 +1479,7 @@ void Widget::applyFilterUpdate(bool force) {
 }
 
 void Widget::searchInChat(Key chat) {
-	onCancelSearch();
+	cancelSearch();
 	setSearchInChat(chat);
 	applyFilterUpdate(true);
 }
@@ -1450,7 +1512,7 @@ void Widget::setSearchInChat(Key chat, PeerData *from) {
 	}
 	_inner->searchInChat(_searchInChat, _searchFromAuthor);
 	if (_searchFromAuthor && _lastFilterText == SwitchToChooseFromQuery()) {
-		onCancelSearch();
+		cancelSearch();
 	}
 	_filter->setFocus();
 }
@@ -1466,9 +1528,9 @@ void Widget::clearSearchCache() {
 	cancelSearchRequest();
 }
 
-void Widget::showJumpToDate() {
+void Widget::showCalendar() {
 	if (_searchInChat) {
-		controller()->showJumpToDate(_searchInChat, QDate());
+		controller()->showCalendar(_searchInChat, QDate());
 	}
 }
 
@@ -1486,15 +1548,15 @@ void Widget::showSearchFrom() {
 	}
 }
 
-void Widget::onFilterCursorMoved(int from, int to) {
+void Widget::filterCursorMoved(int from, int to) {
 	if (to < 0) to = _filter->cursorPosition();
 	QString t = _filter->getLastText();
-	QStringRef r;
+	QStringView r;
 	for (int start = to; start > 0;) {
 		--start;
 		if (t.size() <= start) break;
 		if (t.at(start) == '#') {
-			r = t.midRef(start, to - start);
+			r = base::StringViewMid(t, start, to - start);
 			break;
 		}
 		if (!t.at(start).isLetterOrNumber() && t.at(start) != '_') break;
@@ -1502,14 +1564,16 @@ void Widget::onFilterCursorMoved(int from, int to) {
 	_inner->onHashtagFilterUpdate(r);
 }
 
-void Widget::onCompleteHashtag(QString tag) {
+void Widget::completeHashtag(QString tag) {
 	QString t = _filter->getLastText(), r;
 	int cur = _filter->cursorPosition();
 	for (int start = cur; start > 0;) {
 		--start;
 		if (t.size() <= start) break;
 		if (t.at(start) == '#') {
-			if (cur == start + 1 || t.midRef(start + 1, cur - start - 1) == tag.midRef(0, cur - start - 1)) {
+			if (cur == start + 1
+				|| base::StringViewMid(t, start + 1, cur - start - 1)
+					== base::StringViewMid(tag, 0, cur - start - 1)) {
 				for (; cur < t.size() && cur - start - 1 < tag.size(); ++cur) {
 					if (t.at(cur) != tag.at(cur - start - 1)) break;
 				}
@@ -1537,7 +1601,7 @@ void Widget::updateLockUnlockVisibility() {
 	if (_a_show.animating()) {
 		return;
 	}
-	const auto hidden = !Global::LocalPasscode();
+	const auto hidden = !session().domain().local().hasLocalPasscode();
 	if (_lockUnlock->isHidden() != hidden) {
 		_lockUnlock->setVisible(!hidden);
 		updateControlsGeometry();
@@ -1595,7 +1659,7 @@ void Widget::updateControlsGeometry() {
 	auto smallLayoutWidth = (st::dialogsPadding.x() + st::dialogsPhotoSize + st::dialogsPadding.x());
 	auto smallLayoutRatio = (width() < st::columnMinimalWidthLeft) ? (st::columnMinimalWidthLeft - width()) / float64(st::columnMinimalWidthLeft - smallLayoutWidth) : 0.;
 	auto filterLeft = (controller()->filtersWidth() ? st::dialogsFilterSkip : st::dialogsFilterPadding.x() + _mainMenuToggle->width()) + st::dialogsFilterPadding.x();
-	auto filterRight = (Global::LocalPasscode() ? (st::dialogsFilterPadding.x() + _lockUnlock->width()) : st::dialogsFilterSkip) + st::dialogsFilterPadding.x();
+	auto filterRight = (session().domain().local().hasLocalPasscode() ? (st::dialogsFilterPadding.x() + _lockUnlock->width()) : st::dialogsFilterSkip) + st::dialogsFilterPadding.x();
 	auto filterWidth = qMax(width(), st::columnMinimalWidthLeft) - filterLeft - filterRight;
 	auto filterAreaHeight = st::topBarHeight;
 	_searchControls->setGeometry(0, filterAreaTop, width(), filterAreaHeight);
@@ -1623,7 +1687,7 @@ void Widget::updateControlsGeometry() {
 	auto scrollTop = filterAreaTop + filterAreaHeight;
 	auto newScrollTop = _scroll->scrollTop() + _topDelta;
 	auto scrollHeight = height() - scrollTop;
-	const auto putBottomButton = [&](object_ptr<BottomButton> &button) {
+	const auto putBottomButton = [&](auto &button) {
 		if (button && !button->isHidden()) {
 			const auto buttonHeight = button->height();
 			scrollHeight -= buttonHeight;
@@ -1635,7 +1699,13 @@ void Widget::updateControlsGeometry() {
 		}
 	};
 	putBottomButton(_updateTelegram);
+	putBottomButton(_downloadBar);
 	putBottomButton(_loadMoreChats);
+	const auto bottomSkip = (height() - scrollTop) - scrollHeight;
+	if (_connecting) {
+		_connecting->setBottomSkip(bottomSkip);
+	}
+	controller()->setConnectingBottomSkip(bottomSkip);
 	auto wasScrollHeight = _scroll->height();
 	_scroll->setGeometry(0, scrollTop, width(), scrollHeight);
 	_inner->resize(width(), _inner->height());
@@ -1645,23 +1715,28 @@ void Widget::updateControlsGeometry() {
 	if (_topDelta) {
 		_scroll->scrollToY(newScrollTop);
 	} else {
-		onListScroll();
+		listScrollUpdated();
 	}
 	if (_scrollToTopIsShown) {
 		updateScrollUpPosition();
 	}
 }
 
+rpl::producer<> Widget::closeForwardBarRequests() const {
+	return _closeForwardBarRequests.events();
+}
+
 void Widget::updateForwardBar() {
 	auto selecting = controller()->selectingPeer();
-	auto oneColumnSelecting = (Adaptive::OneColumn() && selecting);
+	auto oneColumnSelecting = (controller()->adaptive().isOneColumn()
+		&& selecting);
 	if (!oneColumnSelecting == !_forwardCancel) {
 		return;
 	}
 	if (oneColumnSelecting) {
 		_forwardCancel.create(this, st::dialogsForwardCancel);
-		_forwardCancel->setClickedCallback([] {
-			Global::RefPeerChooseCancel().notify(true);
+		_forwardCancel->setClickedCallback([=] {
+			_closeForwardBarRequests.fire({});
 		});
 		if (!_a_show.animating()) _forwardCancel->show();
 	} else {
@@ -1671,6 +1746,14 @@ void Widget::updateForwardBar() {
 	update();
 }
 
+RowDescriptor Widget::resolveChatNext(RowDescriptor from) const {
+	return _inner->resolveChatNext(from);
+}
+
+RowDescriptor Widget::resolveChatPrevious(RowDescriptor from) const {
+   return _inner->resolveChatPrevious(from);
+}
+
 void Widget::keyPressEvent(QKeyEvent *e) {
 	if (e->key() == Qt::Key_Escape) {
 		if (_openedFolder) {
@@ -1687,7 +1770,7 @@ void Widget::keyPressEvent(QKeyEvent *e) {
 				_inner->selectSkip(1);
 				_inner->chooseRow();
 			} else {
-				onSearchMessages();
+				searchMessages();
 			}
 		}
 	} else if (e->key() == Qt::Key_Down) {
@@ -1770,17 +1853,15 @@ void Widget::cancelSearchRequest() {
 		base::take(_searchInHistoryRequest));
 }
 
-bool Widget::onCancelSearch() {
+bool Widget::cancelSearch() {
 	bool clearing = !_filter->getLastText().isEmpty();
 	cancelSearchRequest();
 	if (_searchInChat && !clearing) {
-		if (Adaptive::OneColumn()) {
+		if (controller()->adaptive().isOneColumn()) {
 			if (const auto peer = _searchInChat.peer()) {
 				Ui::showPeerHistory(peer, ShowAtUnreadMsgId);
-			//} else if (const auto feed = _searchInChat.feed()) { // #feed
-			//	controller()->showSection(std::make_shared<HistoryFeed::Memento>(feed));
 			} else {
-				Unexpected("Empty key in onCancelSearch().");
+				Unexpected("Empty key in cancelSearch().");
 			}
 		}
 		setSearchInChat(Key());
@@ -1793,37 +1874,32 @@ bool Widget::onCancelSearch() {
 	return clearing;
 }
 
-void Widget::onCancelSearchInChat() {
+void Widget::cancelSearchInChat() {
 	cancelSearchRequest();
+	const auto isOneColumn = controller()->adaptive().isOneColumn();
 	if (_searchInChat) {
-		if (Adaptive::OneColumn()
+		if (isOneColumn
 			&& !controller()->selectingPeer()
 			&& _filter->getLastText().trimmed().isEmpty()) {
 			if (const auto peer = _searchInChat.peer()) {
 				Ui::showPeerHistory(peer, ShowAtUnreadMsgId);
-			//} else if (const auto feed = _searchInChat.feed()) { // #feed
-			//	controller()->showSection(std::make_shared<HistoryFeed::Memento>(feed));
 			} else {
-				Unexpected("Empty key in onCancelSearchInPeer().");
+				Unexpected("Empty key in cancelSearchInPeer().");
 			}
 		}
 		setSearchInChat(Key());
 	}
 	applyFilterUpdate(true);
-	if (!Adaptive::OneColumn() && !controller()->selectingPeer()) {
-		emit cancelled();
-	}
-}
-
-void Widget::onDialogMoved(int movedFrom, int movedTo) {
-	int32 st = _scroll->scrollTop();
-	if (st > movedTo && st < movedFrom) {
-		_scroll->scrollToY(st + st::dialogsRowHeight);
+	if (!isOneColumn && !controller()->selectingPeer()) {
+		controller()->content()->dialogsCancelled();
 	}
 }
 
 Widget::~Widget() {
 	cancelSearchRequest();
+
+	// Destructor may hide the bar and attempt to double-destroy it.
+	base::take(_downloadBar);
 }
 
 } // namespace Dialogs
